<?php

namespace spec\Pim\Bundle\VersioningBundle\EventSubscriber;

use Akeneo\Bundle\BatchBundle\Entity\JobExecution;
use Akeneo\Bundle\BatchBundle\Entity\JobInstance;
use Akeneo\Bundle\BatchBundle\Event\EventInterface;
use Akeneo\Bundle\BatchBundle\Event\JobExecutionEvent;
use PhpSpec\ObjectBehavior;
use Pim\Bundle\VersioningBundle\Manager\VersionContext;
use Prophecy\Argument;

class AddContextSubscriberSpec extends ObjectBehavior
{
    function let(
        VersionContext $versionContext,
        JobExecutionEvent $event,
        JobExecution $jobExecution,
        JobInstance $jobInstance
    ) {
        $this->beConstructedWith($versionContext);

        $event->getJobExecution()->willReturn($jobExecution);
        $jobExecution->getJobInstance()->willReturn($jobInstance);
    }

    function it_is_an_event_listener()
    {
        $this->shouldImplement('Symfony\Component\EventDispatcher\EventSubscriberInterface');
    }

    function it_subscribes_to_the_before_job_execution_event()
    {
        $this->getSubscribedEvents()->shouldReturn([EventInterface::BEFORE_JOB_EXECUTION => 'addContext']);
    }

    function it_injects_versioning_context_into_the_version_manager($event, $jobInstance, $versionContext)
    {
        $jobInstance->getType()->willReturn(JobInstance::TYPE_IMPORT);
        $jobInstance->getCode()->willReturn('foo');

        $versionContext->addContextInfo('import "foo"')->shouldBeCalled();

        $this->addContext($event);
    }

    function it_does_not_inject_context_if_the_job_is_not_an_import($event, $jobInstance, $versionContext)
    {
        $jobInstance->getType()->willReturn(JobInstance::TYPE_EXPORT);

        $versionContext->addContextInfo(Argument::any())->shouldNotBeCalled();

        $this->addContext($event);
    }
}
